#include <stdio.h>
#include <unistd.h>
#include <pthread.h>

typedef struct {
  pthread_cond_t cond;
  pthread_mutex_t mutex;
  volatile unsigned msg;
  // Simple convention: 0 means "no message yet", a message is +ve. In practice,
  // linked list or other data structures, receiver waits for eg non-empty.
} box;

void *sender(void *arg)
{
  box *p = arg;
  sleep(3);
  pthread_mutex_lock(&p->mutex);
  p->msg = 42;
  pthread_mutex_unlock(&p->mutex);
  pthread_cond_signal(&p->cond);
  return NULL;
}

void *receiver(void *arg)
{
  box *p = arg;
  int m;

  printf("Receiver begins waiting for msg\n");
  pthread_mutex_lock(&p->mutex);
  while (p->msg == 0) {
    pthread_cond_wait(&p->cond, &p->mutex);
  }
  m = p->msg;
  p->msg = 0;
  pthread_mutex_unlock(&p->mutex);
  printf("The waiting is over! The message is %d\n", m);
  return NULL;
}

int main(void)
{
  pthread_t t1, t2;
  box b;

  b.msg = 0;
  pthread_cond_init(&b.cond, NULL);
  pthread_mutex_init(&b.mutex, NULL);

  pthread_create(&t1, NULL, sender, &b);
  pthread_create(&t2, NULL, receiver, &b);

  pthread_join(t1, NULL);
  pthread_join(t2, NULL);

  pthread_mutex_destroy(&b.mutex);
  pthread_cond_destroy(&b.cond);

  return 0;
}
