#include <stdio.h>
#include <unistd.h>
#include <pthread.h>

pthread_cond_t cond = PTHREAD_COND_INITIALIZER;
pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
volatile unsigned msg = 0;
// Simple convention: 0 means "no message yet", a message is +ve. In practice,
// linked list or other data structures, consumer waits for eg non-empty.

void *sender(void *arg)
{
  for (unsigned i = 40; i < 50; i++) {
    sleep(3);
    pthread_mutex_lock(&mutex);
    msg = i;
    pthread_mutex_unlock(&mutex);
    pthread_cond_signal(&cond);
  }
  return NULL;
}

void *consumer(void *arg)
{
  const char *myname = arg;
  unsigned m;

  for (;;) {
    printf("%s waits for msg\n", myname);
    pthread_mutex_lock(&mutex);
    while (msg == 0) {
      pthread_cond_wait(&cond, &mutex);
    }
    // Why while instead of if: Either way OK for this toy example, but better
    // habit to use while, gives you more freedom in case:
    // * different consumers waiting for different predicates
    // * sender notifies everyone upon any change because it isn't sure relevant
    //   to whom
    // * so you are notified but your predicate is still false, should wait again
    m = msg;
    msg = 0;
    pthread_mutex_unlock(&mutex);
    printf("%s gets message %u\n", myname, m);
  }
  return NULL;
}

int main(void)
{
  pthread_t tsender, tconsumer1, tconsumer2;

  pthread_create(&tsender, NULL, sender, NULL);
  pthread_create(&tconsumer1, NULL, consumer, "A");
  pthread_create(&tconsumer2, NULL, consumer, "B");

  pthread_join(tsender, NULL);
  sleep(1);  // Give consumers some time.

  return 0;
}
